package com.loafle.overflow.central.module.email.service;

import com.loafle.overflow.central.commons.model.Mail;
import com.loafle.overflow.central.commons.utils.EmailSender;
import com.loafle.overflow.central.module.domain.dao.DomainDAO;
import com.loafle.overflow.central.module.domain.dao.DomainMemberDAO;
import com.loafle.overflow.central.module.domain.model.Domain;
import com.loafle.overflow.central.module.domain.model.DomainMember;
import com.loafle.overflow.central.module.email.dao.EmailAuthDAO;
import com.loafle.overflow.central.module.email.exception.EmailOverAuthException;
import com.loafle.overflow.central.module.email.model.EmailAuth;
import com.loafle.overflow.central.module.member.dao.MemberDAO;
import com.loafle.overflow.central.module.member.model.Member;
import com.loafle.overflow.central.module.meta.model.MetaMemberStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;

/**
 * Created by geek on 17. 6. 28.
 */
@Service("EmailAuthService")
public class EmailAuthService {

    @Autowired
    private EmailAuthDAO emailAuthDAO;

    @Autowired
    private MemberDAO memberDAO;

    @Autowired
    private DomainDAO domainDAO;

    @Autowired
    private DomainMemberDAO domainMemberDAO;

    @Autowired
    private EmailSender emailSender;


    public EmailAuth sendEmailByMember(long memberId, String memberEmail) throws UnsupportedEncodingException, MailException {

        EmailAuth auth = new EmailAuth();
        auth.setMember(new Member(memberId));
        // Todo AuthKey Generation

        String en = emailSender.encrypt(memberEmail);
        auth.setEmailAuthKey(en);

        String encode = URLEncoder.encode(en, "UTF-8");

//        System.out.println("encode = [" + encode + "]");
        Mail mail = new Mail();
        mail.setMailTo(memberEmail);
        mail.setMailSubject("Confirm Email");
        mail.setMailContent("http://127.0.0.1:19080/account/check_email?key="+ encode +"\r\nConfirm Email");
        Map<String, Object> model = new HashMap<>();
        model.put("firstName", auth.getMember().getName());
        model.put("lastName", auth.getMember().getCompanyName());
        model.put("location", "Seoul");
        model.put("signature", "www.loafle.com");

        mail.setModel(model);
        emailSender.sendSimpleEmail(mail);

        this.emailAuthDAO.save(auth);

        return auth;
    }

    public EmailAuth read(long id) {
        return this.emailAuthDAO.findOne(id);
    }

    public EmailAuth readByAuthKey(String authKey) throws UnsupportedEncodingException {
        System.out.println("authKey = [" + authKey + "]");
        String deStr = URLDecoder.decode(authKey, "UTF-8");
        System.out.println("deStr = [" + deStr + "]");

        EmailAuth auth = this.emailAuthDAO.findByEmailAuthKey(deStr);

        // Todo Compare email date and current date

        if (auth != null) {

            // Over 12 hours of validity of e-mail authentication.
            Calendar cal = Calendar.getInstance();

            cal.setTime(auth.getCreateDate());
            cal.add(Calendar.HOUR, 12);
            Date futureDate = cal.getTime();

            Date nowDate = new Date();

            if (!nowDate.before(futureDate)) {
                throw new EmailOverAuthException("The authentication expiration time has passed.");
            }

            auth.setAuthConfirmDate(new Date());
            this.emailAuthDAO.save(auth);
            auth.getMember().setStatus(new MetaMemberStatus((short)2));
            this.memberDAO.save(auth.getMember());

            Domain domain = new Domain();
            domain.setName(auth.getMember().getCompanyName());
            Domain domain1 = this.domainDAO.save(domain);

            DomainMember domainMember = new DomainMember();
            domainMember.setDomain(domain1);
            domainMember.setMember(auth.getMember());
            this.domainMemberDAO.save(domainMember);
        }

        return auth;
    }

    // dZQgXM1o/Cx48X8DM+6ec/oPfqA2l/LdWtijOZ2EnWk=
    // dZQgXM1o/Cx48X8DM 6ec/oPfqA2l/LdWtijOZ2EnWk=

    public List<EmailAuth> readByMember(long memberId) {
        return this.emailAuthDAO.findByMember(new Member(memberId));
    }

    public EmailAuth modify(EmailAuth emailAuth) {
        return this.emailAuthDAO.save(emailAuth);
    }

    // Todo Send Email Refactoring
    public EmailAuth sendEmailResetPassword(Member member) throws UnsupportedEncodingException, MailException {
        EmailAuth auth = new EmailAuth();
        auth.setMember(member);
        // Todo AuthKey Generation

        String en = emailSender.encrypt(member.getEmail());
        auth.setEmailAuthKey(en);

        String encode = URLEncoder.encode(en, "UTF-8");

//        System.out.println("encode = [" + encode + "]");
        Mail mail = new Mail();
        mail.setMailTo(member.getEmail());
        mail.setMailSubject("Reset Password Email");
        mail.setMailContent("http://127.0.0.1:9091/#/account/reset_password?key="+ encode +"\r\nConfirm Email");
        Map<String, Object> model = new HashMap<>();
        model.put("firstName", auth.getMember().getName());
        model.put("lastName", auth.getMember().getCompanyName());
        model.put("location", "Seoul");
        model.put("signature", "www.loafle.com");

        mail.setModel(model);
        emailSender.sendSimpleEmail(mail);

        this.emailAuthDAO.save(auth);

        return auth;
    }


}
