package com.loafle.overflow.central.module.noauthprobe.service;

import com.loafle.overflow.central.commons.service.MessagePublisher;
import com.loafle.overflow.central.commons.stereotype.ProbeAPI;
import com.loafle.overflow.central.commons.stereotype.WebappAPI;
import com.loafle.overflow.central.commons.utils.GenerateKey;
import com.loafle.overflow.central.module.noauthprobe.dao.NoAuthProbeDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.apikey.ApiKey;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.domain.DomainMember;
import com.loafle.overflow.model.member.Member;
import com.loafle.overflow.model.meta.MetaNoAuthProbeStatus;
import com.loafle.overflow.model.meta.MetaProbeStatus;
import com.loafle.overflow.model.noauthprobe.NoAuthProbe;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.service.central.apikey.ApiKeyService;
import com.loafle.overflow.service.central.domain.DomainMemberService;
import com.loafle.overflow.service.central.noauthprobe.NoAuthProbeService;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by snoop on 17. 6. 28.
 */
@Service("NoAuthProbeService")
public class CentralNoAuthProbeService implements NoAuthProbeService{

    @Autowired
    private NoAuthProbeDAO noAuthProbeDAO;

    @Autowired
    private ApiKeyService apiKeyService;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private MessagePublisher messagePublisher;

    @Autowired
    private DomainMemberService domainMemberService;

    @ProbeAPI
    public NoAuthProbe regist(NoAuthProbe noAuthProbe) throws OverflowException{

        ApiKey apiKey = apiKeyService.readByApiKey(noAuthProbe.getApiKey());
        noAuthProbe.setDomain(apiKey.getDomain());

        noAuthProbe.setTempProbeKey(GenerateKey.getKey());
        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short)3));

        messagePublisher.publishToDomainMembers(apiKey.getDomain().getId(), "NoAuthProbeService.regist", noAuthProbe);

        return this.noAuthProbeDAO.save(noAuthProbe);
    }


    public List<NoAuthProbe> readAllByDomain(Domain domain) throws OverflowException {

        return this.noAuthProbeDAO.findAllByDomain(domain);
    }

    public NoAuthProbe read(long id) {
        return this.noAuthProbeDAO.findOne(id);
    }

    @WebappAPI
    public List<NoAuthProbe> acceptNoAuthProbe(NoAuthProbe noAuthProbe) throws IOException {
        String memberEmail = SessionMetadata.getTargetID();

        // Todo domain injection & member injection

        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        String encryptKey = passwordEncoder.encode(UUID.randomUUID().toString());

        ApiKey apiKey = apiKeyService.readByApiKey(noAuthProbe.getApiKey());
        DomainMember domainMember = domainMemberService.readByMemberEmail(memberEmail);

        Probe probe = new Probe();
        probe.setEncryptionKey(encryptKey);
        probe.setProbeKey(GenerateKey.getKey());

        probe.setDomain(new Domain(apiKey.getDomain().getId()));
        probe.setAuthorizeMember(new Member(domainMember.getMember().getId()));
        probe.setStatus(new MetaProbeStatus((short)1));

        Map<String, Object> objMap = this.objectMapper.readValue(noAuthProbe.getDescription(), new TypeReference<HashMap<String,Object>>() {});
        Map<String, String> hostMap = (Map<String, String>)objMap.get("host");
        Map<String, String> netMap = (Map<String, String>)objMap.get("network");

//            String dispName = noAuth.getHostName().isEmpty() ?
//                    StringConvertor.intToIp(noAuth.getIpAddress()) : noAuth.getHostName();
        String dispName = hostMap.get("name");
        dispName += " probe";
        probe.setDisplayName(dispName);

        String addrStr = netMap.get("address");

        String[] addrArr = addrStr.split("\\|");

        probe.setCidr(addrArr[0]);

        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short) 1));

        this.probeService.regist(probe);
        this.noAuthProbeDAO.save(noAuthProbe);

        messagePublisher.publishToNoAuthProbe(noAuthProbe.getTempProbeKey(), "NoAuthProbeService.Accept", probe.getProbeKey());

        return this.readAllByDomain(noAuthProbe.getDomain());
    }

    @WebappAPI
    public List<NoAuthProbe> denyNoauthProbe(NoAuthProbe noAuthProbe) throws OverflowException {
        noAuthProbe.setStatus(new MetaNoAuthProbeStatus((short) 2));
        this.noAuthProbeDAO.save(noAuthProbe);

        messagePublisher.publishToNoAuthProbe(noAuthProbe.getTempProbeKey(), "NoAuthProbeService.Deny");

        return this.readAllByDomain(noAuthProbe.getDomain());
    }

    public NoAuthProbe readByTempKey(String tempKey) {
        return this.noAuthProbeDAO.findByTempProbeKey(tempKey);
    }
}
