package com.loafle.overflow.central.module.sensor.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.generator.service.SensorConfigGenerator;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.domain.Domain;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.meta.MetaSensorStatus;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.sensor.SensorItem;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.probe.ProbeService;
import com.loafle.overflow.service.central.sensor.SensorService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.io.IOException;
import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class CentralSensorService implements SensorService {

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private InfraService infraService;

    @Autowired
    private CentralSensorItemService sensorItemService;

    @Autowired
    private SensorConfigGenerator sensorConfigGenerator;

    public Sensor regist(Sensor sensor) {
        return this.sensorDAO.save(sensor);
    }

    public Page<Sensor> readAllByDomain(Domain domain, PageParams pageParams) throws OverflowException {

        List<Probe> probeList = this.probeService.readAllByDomain(domain);

        if (probeList == null || probeList.size() <= 0) {
            throw new OverflowException("", new Throwable());
        }

        List<Target> targetList = this.infraService.readAllTargetByProbeList(probeList);

        if (targetList == null || targetList.size() <= 0) {
            throw new OverflowException("", new Throwable());
        }

        return this.sensorDAO.findAllByTargetList(targetList, PageUtil.getPageRequest(pageParams));
    }

    public Page<Sensor> readAllByInfra(long infraId, PageParams pageParams) throws OverflowException {
        Infra dbInfra = this.infraService.read(infraId);

        if (dbInfra == null || dbInfra.getTarget() == null) {
            throw new InfraNotFoundException();
        }

        return this.sensorDAO.findAllByTarget(dbInfra.getTarget(), PageUtil.getPageRequest(pageParams));
    }

    public Page<Sensor> readAllByTarget(Target target, PageParams pageParams) throws OverflowException {
        return this.sensorDAO.findAllByTarget(target, PageUtil.getPageRequest(pageParams));
    }

    public Sensor read(String id) throws OverflowException {
        return this.sensorDAO.findOne(Long.valueOf(id));
    }

    public void remove(Sensor sensor) throws OverflowException {
        this.sensorDAO.delete(sensor);
    }

    public Sensor start(Sensor sensor) throws OverflowException {
        MetaSensorStatus status = new MetaSensorStatus((short) 1);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    public Sensor stop(Sensor sensor) throws OverflowException {
        MetaSensorStatus status = new MetaSensorStatus((short) 2);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    @Transactional
    public Sensor registSensorConfig(Sensor sensor, List<SensorItem> sensorItemList, String etcJson) throws OverflowException {

        this.sensorDAO.save(sensor);

        for (SensorItem sensorItem : sensorItemList) {
            sensorItem.setSensor(sensor);
        }

        this.sensorItemService.registAll(sensorItemList);

        return sensor;
    }

    public String generateSensorConfig(Sensor sensor) throws OverflowException {

        return this.sensorConfigGenerator.generate(sensor);

    }

    //    public List<Sensor> readAllByTarget(Target target, PageParams pageParams) {
    //        return this.sensorDAO.findAllByTarget(target, PageUtil.getPageRequest(pageParams));
    //    }
}
